@@ -6,7 +6,7 @@
  *  Updated   : 2010-04-29
  *  Notes     : nntp.h 1.5.11/1.6 with extensions for tin
  *
- * Copyright (c) 1991-2011 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2014 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,14 +45,6 @@
 #define NNTP_TCP_NAME	"nntp"
 #define NNTP_TCP_PORT	"119"
 
-/*
- * # seconds after which a read from the NNTP will timeout
- * NB: This is different from the NNTP server timing us out due to inactivity
- */
-#if defined(HAVE_ALARM) && defined(SIGALRM)
-#	define NNTP_READ_TIMEOUT	120
-#endif /* HAVE_ALARM && SIGALRM */
-
 /*
  * # times to try and reconnect to server after timeout
  */
@@ -173,7 +165,7 @@ struct t_capabilities {
 	enum extension_type type;		/* NONE, CAPABILITIES, BROKEN */
 	unsigned int version;			/* CAPABILITIES version */
 	t_bool mode_reader:1;			/* MODE-READER: "MODE READER" */
-	t_bool reader:1;				/* READER: "ARTCILE", "BODY", "DATE", "GROUP", "LAST", "LISTGROUP", "NEWGROUPS", "NEXT" */
+	t_bool reader:1;				/* READER: "ARTICLE", "BODY", "DATE", "GROUP", "LAST", "LISTGROUP", "NEWGROUPS", "NEXT" */
 	t_bool post:1;					/* POST */
 	t_bool list_active:1;			/* LIST ACTIVE */
 	t_bool list_active_times:1;		/* LIST ACTIVE.TIMES, optional */
