@@ -3,10 +3,10 @@
  *  Module    : lock.c
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   : 1998-07-27
- *  Updated   : 2006-05-11
+ *  Updated   : 2013-11-17
  *  Notes     :
  *
- * Copyright (c) 1998-2012 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1998-2014 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -213,7 +213,8 @@ fd_unlock(
  *  TRUE  = file locked successfully
  *  FALSE = some error occurred
  */
-t_bool dot_lock(
+t_bool
+dot_lock(
 	const char *filename)
 {
 	char tempfile[PATH_LEN];
@@ -226,12 +227,12 @@ t_bool dot_lock(
 	dir_name(filename, base_dir);
 	if (!strcmp(filename, base_dir)) /* no filename portion */
 		return rval;
-	if ((dot_fd = my_tmpfile(tempfile, sizeof(tempfile) - 1, TRUE, base_dir)) == -1)
+	if ((dot_fd = my_tmpfile(tempfile, sizeof(tempfile) - 1, base_dir)) == -1)
 		return rval;
 	snprintf(lockfile, sizeof(lockfile), "%s%s", filename, LOCK_SUFFIX);
 
 #ifdef HAVE_LINK
-	if (stat(lockfile, &statbuf)) {				/* lockfile doesn't exist */
+	if (stat(lockfile, &statbuf)) {	/* lockfile doesn't exist */
 		if (!link(tempfile, lockfile)) {			/* link successful */
 			if (!stat(tempfile, &statbuf)) {	/* tempfile exist */
 				if (statbuf.st_nlink == 2)			/* link count ok */
@@ -259,7 +260,8 @@ t_bool dot_lock(
  *  TRUE  = file unlocked successfully
  *  FALSE = some error occurred
  */
-t_bool dot_unlock(
+t_bool
+dot_unlock(
 	const char *filename)
 {
 	char *lockfile;
