@@ -6,7 +6,7 @@
  *  Updated   : 2011-11-06
  *  Notes     : RFC 2046 MIME article parsing
  *
- * Copyright (c) 2000-2012 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 2000-2014 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -209,7 +209,7 @@ get_quoted_string(
 	char *ptr;
 	t_bool quote = FALSE;
 
-	*dest = my_malloc(strlen(source));
+	*dest = my_malloc(strlen(source) + 1);
 	ptr = *dest;
 	source++; /* skip over double quote */
 	while (*source) {
